#########################################################################################################
####### Fitting datasets from Plan et al. 2012 with nlmer

library("lme4")
prefix.data <- "mc-sim-1-"

# Emax model for nlme/nlmer
nformemax <- ~ exp(le0) + exp(lemax) * input / (exp(le50) + input)
nfunemax <- deriv(nformemax, namevec = c("le0", "lemax", "le50"), 
                  function.arg = c("input", "le0", "lemax", "le50"))

# Initial estimates
pinit <- c(ptrue[c(4, 1:2)], 1, ptrue[c(8, 5, 7)], 1, ptrue[6])
vecinit0 <- c(le0 = log(pinit[1]), lemax = log(pinit[2]), le50 = log(pinit[3]))

for (scenar in scenariosPlan) {
  cat("********************************************************\n")
  cat("        Scenario", scenar, "\n")
  cat("********************************************************\n")

  # Datasets
  namdir.data <- file.path(work.dir, "dataPlan12", scenar)
  namres.dir <- paste(scenar, "_nlmer", sep = "")
  namdir <- file.path(dir.fit, namres.dir)
  dir.create(namdir)
  resnlmer <- resnlmer.lscale <- NULL
  
  for (irep in 1:nsim) {
    datnlmer <- read.table(file.path(namdir.data, paste(prefix.data, irep, ".dat", sep = "")), header = TRUE, skip = 1)
    icount <- 0
    ipb <- 1
    repeat {
        if (icount >= 10 | ipb == 0) break
        icount <- icount + 1
        if (icount == 1) startvecemax <- vecinit0 else startvecemax <- vecinit0 * (1 + rnorm(length(vecinit0), sd = 0.2))
        nlmer.fit <- try(nlmer(DV~nfunemax(DOSE, le0, lemax, le50) ~ (le0|ID) + ((lemax + le50)|ID),
                               data = datnlmer, start = list(nlpars = startvecemax, theta = c(pinit[c(5:7)], 0.45))))
        if(class(nlmer.fit) != "try-error") ipb <- 0
    } 
    
    if (ipb == 0) {
        # delta-method to compute SE on parameters (exp(le0), ...)
        vcov.beta <- vcov(nlmer.fit)
        beta <- summary(nlmer.fit)$coef[, 1]
        se.par <- diag(diag(exp(beta)) %*% vcov.beta %*% diag(exp(beta)))
        
        l1 <- c(irep, summary(nlmer.fit)$coef[, 1], attr(summary(nlmer.fit)$varcor$ID, "stddev"),
                attr(summary(nlmer.fit)$varcor$ID.1, "stddev"), attr(summary(nlmer.fit)$varcor$ID.1, "correlation")[1, 2],
                summary(nlmer.fit)$sigma, summary(nlmer.fit)$coef[, 2],  AIC(nlmer.fit), as.double(logLik(nlmer.fit)), icount)
        resnlmer.lscale <- rbind(resnlmer.lscale, l1)
        l2 <- c(irep, exp(summary(nlmer.fit)$coef[, 1]), attr(summary(nlmer.fit)$varcor$ID, "stddev"),
                attr(summary(nlmer.fit)$varcor$ID.1, "stddev"), attr(summary(nlmer.fit)$varcor$ID.1, "correlation")[1, 2],
                summary(nlmer.fit)$sigma, se.par, AIC(nlmer.fit), as.double(logLik(nlmer.fit)), icount)
        resnlmer <- rbind(resnlmer, l2)
    } else {
      resnlmer.lscale <- rbind(resnlmer.lscale, c(irep, rep(NA, 14)))
      resnlmer <- rbind(resnlmer, c(irep, rep(NA, 14)))
    }
  }
  colnames(resnlmer) <- c("numsim", "e0", "emax", "e50", "sd.le0", "sd.lemax", "sd.le50", "corr.lemax.le50", "sigma", "se.e0", "se.emax", "se.e50", "AIC", "LL", "ntry")
  colnames(resnlmer.lscale) <- c("numsim", "le0", "lemax", "le50", "sd.le0", "sd.lemax", "sd.le50", "corr.lemax.le50", "sigma", "se.le0", "se.lemax", "se.le50", "AIC", "LL", "ntry")
  
  # Saving results
  write.table(resnlmer.lscale, file.path(namdir, "resnlmer_logscale.res"), quote = FALSE, row.names = FALSE)
  write.table(resnlmer, file.path(namdir, "resnlmer_origscale.res"), quote = FALSE, row.names = FALSE)

}

##################
# Checking delta-method for estimates of SE(parameters) from SE(log-parameters)
library("msm")
deltamethod(list(~exp(x1), ~exp(x2), ~exp(x3)), mean = beta, cov = vcov.beta, ses = FALSE)
diag(exp(beta)) %*% vcov.beta %*% diag(exp(beta))
