#########################################################################################################
####### Fitting datasets from Plan et al. 2012 with saemix

for (scenar in scenariosPlan) {
  for (settings in fitsettings) {
    cat("********************************************************\n")
    cat("        Scenario",scenar,"\n")
    cat("********************************************************\n")
    # Initial parameters for estimation
    if (!is.na(pmatch("ptrue", settings))) {
      ci.param <- ptrue
      ci.param[3] <- as.integer(substr(scenar, 2, 2))
      } else {
      ci.param <- pwrong
      ci.param[3] <- 1
    }
    # Residual error model
    #    ires <- ifelse(length(grep("P",scenar)) > 0, 2, 1)
    #    if (length(grep("P", scenar)) > 0) {ptrue[9] <- 0.01; pwrong[9] <- 0.0625} else {ptrue[9] <- 4; pwrong[9] <- 1}
    sigma <- sqrt(ptrue[9])
    ires <- 1 # Fitting additive model only
    if (length(grep("A", scenar)) > 0) {
      error.model <- "constant"
      error.init <- c(sqrt(ci.param[9]), 0)
    } else {
      error.model <- "proportional"
      error.init <- c(0, sqrt(ci.param[9]))
    }
    # Initial estimates for IIV
    omega.init <- diag(c(ci.param[c(8, 5, 7)], 1))
    omega.init[2, 3] <- omega.init[3, 2] <- ci.param[6]
    
    namres.dir <- paste(scenar, "_", settings, sep = "")
    namdir <- file.path(dir.fit, namres.dir)
    dir.create(namdir)
    namdir.data <- file.path(work.dir, "dataPlan12", scenar)

    # Saemix model components
    if (length(grep("default", settings)) > 0) {
        saemix.options <- list(directory = namdir, save = FALSE,save.graphs = FALSE, displayProgress = FALSE)
    } else {
        saemix.options <- list(directory = namdir, nb.chains = 5, nbiter.saemix = c(300, 150),
                               save = FALSE, save.graphs = FALSE, displayProgress = FALSE)
    }
    
    emax.model <- saemixModel(model = modelhill, description = "Emax model with sigmoidicity (Hill model)",
                              psi0 = matrix(c(ci.param[c(4, 1:2, 3)], 0, 0, 0, 0), ncol = 4, byrow = TRUE,
                                            dimnames = list(NULL, c("E0", "Emax", "EC50", "gamma"))), transform.par = c(1, 1, 1, 1),
                              covariate.model = matrix(c(0, 0, 0, 0), ncol = 4, byrow = TRUE),
                              covariance.model = matrix(c(1, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 0, 0, 0), ncol = 4, byrow = TRUE),
                              omega.init = omega.init, error.init = error.init, error.model = error.model)
    
    # Loop in simulations
    logl <- par.nocov <- se.nocov <- NULL
    
    for (irep in 1:nsim) {
      dat <- read.table(file.path(namdir.data, paste(prefix.data, irep, ".dat", sep = "")), header = TRUE, skip = 1)
      dat <- cbind(dat, treat = ifelse(dat$ID < 50, 0, 1))
      dat <- saemixData(name.data = dat, header = TRUE, name.group = "ID",
                        name.predictors = "DOSE", name.response = "DV",
                        name.covariates = c("treat"), units = list(x = "mg", y = "-", covariates = "-"))			
      
      saemix.fit <- saemix(emax.model, dat, saemix.options)
      saemix.fit <- llgq.saemix(saemix.fit)
      namres <- file.path(namdir, paste("results_nocov_", irep, ".txt", sep = ""))
      xtry <- try(sink(namres))
      if (class(xtry) != "try-error") {
        print(saemix.fit@results)
        sink()
      }
      # Saving the results to a file
      l1 <- c(irep, saemix.fit@results@ll.lin, saemix.fit@results@ll.is, saemix.fit@results@ll.gq)
      logl <- rbind(logl, l1)
      yres <- saemix.fit@results
      rho <- cov2cor(yres@omega[1:3, 1:3])[3, 2]
      l1 <- c(irep, saemix.fit@options$seed, yres@fixed.effects, yres@respar[ires], diag(yres@omega)[1:3], yres@omega[3, 2], rho)
      par.nocov <- rbind(par.nocov, l1)
      l1 <- c(irep, yres@se.fixed, yres@se.respar[ires], yres@se.omega[1:3])
      se.nocov <- rbind(se.nocov, l1) # missing the SE for the covariance term  
    }
    colnames(logl) <- c("Replication", "LLlin.nocov", "LLis.nocov", "LLgq.nocov")
    colnames(par.nocov) <- c("Replication", "seed", saemix.fit@results@name.fixed, saemix.fit@results@name.res[ires], saemix.fit@results@name.random, "cov.Emx.EC50", "rho")
    colnames(se.nocov) <- c("Replication", paste("SE", c(saemix.fit@results@name.fixed, saemix.fit@results@name.res[ires], saemix.fit@results@name.random), sep = "."))
    # Saving results
    write.table(logl, file.path(namdir, "LL.res"), quote = FALSE, row.names = FALSE)
    write.table(par.nocov, file.path(namdir, "parnocov.res"), quote = FALSE, row.names = FALSE)
    write.table(se.nocov, file.path(namdir, "senocov.res"), quote = FALSE, row.names = FALSE)
  }
}
